"""

Contains classes related to Roblox friend data and parsing.

"""
from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from .users import User

if TYPE_CHECKING:
    from .client import Client


class Friend(User):
    """
    Represents a friend.

    Attributes:
        is_online: Whether the user is currently online.
        presence_type: Their presence type. Don't use this.
        is_deleted: Whether the account is deleted.
        friend_frequent_rank: Unknown
    """

    def __init__(self, client: Client, data: dict):
        """
        Arguments:
            data: The data we get back from the endpoint.
            client: The Client object, which is passed to all objects this Client generates.
        """
        super().__init__(client=client, data=data)

        self.is_online: Optional[bool] = data.get("isOnline")
        self.presence_type: Optional[int] = data.get("presenceType")
        self.is_deleted: bool = data["isDeleted"]
        self.friend_frequent_rank: int = data["friendFrequentRank"]

    def __repr__(self):
        return f"<{self.__class__.__name__} id={self.id} name={self.name!r} is_online={self.is_online}>"



"""

This file contains the BaseJob object, which represents a Roblox job ID.

"""

from __future__ import annotations
from typing import TYPE_CHECKING

from .baseitem import BaseItem

if TYPE_CHECKING:
    from ..client import Client


class BaseJob(BaseItem):
    """
    Represents Roblox job ID.

    Job IDs are UUIDs that represent a single game server instance.
    Learn more on the Developer Hub [here](https://developer.roblox.com/en-us/api-reference/property/DataModel/JobId).

    Attributes:
        id: The job ID.
    """

    def __init__(self, client: Client, job_id: str):
        """
        Arguments:
            client: The Client this object belongs to.
            job_id: The job ID.
        """

        self._client: Client = client
        self.id: str = job_id



"""

This file contains the BasePlace object, which represents a Roblox place ID.

"""

from __future__ import annotations
from typing import TYPE_CHECKING

from ..bases.baseasset import BaseAsset

if TYPE_CHECKING:
    from ..client import Client


class BasePlace(BaseAsset):
    """
    Represents a Roblox place ID.
    Places are a form of Asset and as such this object derives from BaseAsset.

    Attributes:
        id: The place ID.
    """

    def __init__(self, client: Client, place_id: int):
        """
        Arguments:
            client: The Client this object belongs to.
            place_id: The place ID.
        """

        super().__init__(client, place_id)

        self._client: Client = client
        self.id: int = place_id

    async def get_instances(self, start_index: int = 0):
        """
        Returns a list of this place's current active servers, known in the API as "game instances".
        This list always contains 10 items or fewer.
        For more items, add 10 to the start index and repeat until no more items are available.

        Arguments:
            start_index: Where to start in the server index.
        """
        from ..jobs import GameInstances

        instances_response = await self._client.requests.get(
            url=self._client.url_generator.get_url("games", f"v1/games/{self.id}/servers/Public?sortOrder=Asc&limit=100"),
            params={
                "placeId": self.id,
                "startIndex": start_index
            }
        )
        instances_data = instances_response.json()
        return GameInstances(
            client=self._client,
            data=instances_data,
            place_id=self.id
        )
